/***********************************************************
* Artsoft Retro-Game Library                               *
*----------------------------------------------------------*
*               Artsoft Entertainment                      *
*               Holger Schemel                             *
*               Detmolder Strasse 189                      *
*               33604 Bielefeld                            *
*               Germany                                    *
*               e-mail: info@artsoft.org                   *
*----------------------------------------------------------*
* zutil.c                                                  *
*----------------------------------------------------------*
* (c) by the owner of the alias/nickname "HerzAusGold"     *
*               e-mail: herzausgold@online.de              *
***********************************************************/
/*
 * parts from
 *  miniunz.c
 *  Version 1.01e, February 12th, 2005
 *
 *  Copyright (C) 1998-2005 Gilles Vollant
*/


#include <sys/types.h>
#include <sys/stat.h>
#include <string.h>
#include <fcntl.h>
#include <errno.h>

#include "platform.h"
#include "misc.h"
#include "rndapi.h"		/*#HAG#VC#*/
#include "util.h"


#include "unzip.h"


#define CASESENSITIVITY (0)
#define WRITEBUFFERSIZE (8192)
#define MAXFILENAME (256)


static long dircount  = 0;
static long filecount = 0;
static List_t levelInfoList = {0,0,0,0};  // variable char list


/*----------------------------------------------------------------*/
char * strstr_or_wc (char *curString,
                     char *subString)
{
	if (strchr(subString, '*')) {
		return (strFindWildcard(curString, subString));
	} else {
		return (strstr(curString, subString));
	}
}


/*----------------------------------------------------------------*/
/*-- from
  mini unzip, demo of unzip package

  usage :
  Usage : miniunz [-exvlo] file.zip [file_to_extract] [-d extractdir]

  list the file in the zipfile, and print the content of FILE_ID.ZIP or README.TXT
    if it exists
*/


/* change_file_date : change the date/time of a file
    filename : the filename of the file where date/time must be modified
    dosdate : the new date at the MSDos format (4 bytes)
    tmu_date : the SAME new date at the tm_unz format */
void change_file_date(
    char *filename,
    uLong dosdate,
    tm_unz tmu_date
    )
{
/*--
#if defined(PLATFORM_WIN32)
  HANDLE hFile;
  FILETIME ftm,ftLocal,ftCreate,ftLastAcc,ftLastWrite;

  hFile = CreateFile(filename,GENERIC_READ | GENERIC_WRITE,
                      0,NULL,OPEN_EXISTING,0,NULL);
  GetFileTime(hFile,&ftCreate,&ftLastAcc,&ftLastWrite);
  DosDateTimeToFileTime((WORD)(dosdate>>16),(WORD)dosdate,&ftLocal);
  LocalFileTimeToFileTime(&ftLocal,&ftm);
  SetFileTime(hFile,&ftm,&ftLastAcc,&ftm);
  CloseHandle(hFile);
#else
  struct utimbuf ut;
  struct tm newdate;
  newdate.tm_sec = tmu_date.tm_sec;
  newdate.tm_min=tmu_date.tm_min;
  newdate.tm_hour=tmu_date.tm_hour;
  newdate.tm_mday=tmu_date.tm_mday;
  newdate.tm_mon=tmu_date.tm_mon;
  if (tmu_date.tm_year > 1900)
      newdate.tm_year=tmu_date.tm_year - 1900;
  else
      newdate.tm_year=tmu_date.tm_year ;
  newdate.tm_isdst=-1;

  ut.actime=ut.modtime=mktime(&newdate);
  utime(filename,&ut);
#endif
--*/
}


/* mymkdir and change_file_date are not 100 % portable
   As I don't know well Unix, I wait feedback for the unix portion */

static int mymkdir(
    char* toFolder,
    char* dirname
    )
{
    int ret=0;
#if defined(PLATFORM_WIN32)
#if defined(CYGWIN)
    ret = mkdir(getPath2(toFolder,dirname),0775);
#else
    ret = mkdir(getPath2(toFolder,dirname));
#endif    
#else
    ret = mkdir (getPath2(toFolder,dirname),0775);
#endif
    listEnter(&levelInfoList, dirname, 
              strlen(dirname)+1, TRUE);            
    dircount++;
   
    return ret;
}

static int make_mkdir (
    char *toFolder,
    char *newdir
    )
{
  char *buffer ;
  char *p;
  int  len = (int)strlen(newdir);

  if (len <= 0)
    return 0;

  buffer = (char*)malloc(len+1);
  strcpy(buffer,newdir);

  if (buffer[len-1] == '/') {
    buffer[len-1] = '\0';
  }
  if (mymkdir(toFolder, buffer) == 0) {
      free(buffer);
      return 1;
  }

  p = buffer+1;
  while (1) 
  {
      char hold;

      while(*p && *p != '\\' && *p != '/')
        p++;
      hold = *p;
      *p = 0;
      if ((mymkdir(toFolder, buffer) == -1) && (errno == ENOENT))
      {
          fprintf(stderr,"couldn't create directory %s\n",buffer);
          free(buffer);
          return 0;
      }
      if (hold == 0)
        break;
      *p++ = hold;
  }
  free(buffer);
  return 1;
}


int do_extract_currentfile(
    unzFile uf,
	char* toFolder,
    char* filemask,
    int* popt_extract_without_path,
    int* popt_overwrite,
    char* password)
{
    char filename_inzip[256];
    char* filename_withoutpath;
    char* p;
    int err=UNZ_OK;
    FILE *fout=NULL;
    void* buf;
    uInt size_buf;

    unz_file_info file_info;
//    uLong ratio=0;
    err = unzGetCurrentFileInfo(uf,&file_info,filename_inzip,sizeof(filename_inzip),NULL,0,NULL,0);

    if (err!=UNZ_OK)
    {
        fprintf(stderr,"error %d with zipfile in unzGetCurrentFileInfo\n",err);
        return err;
    }
    if ((filemask != NULL) && (strstr_or_wc(filename_inzip, filemask) == NULL)) {
	    return UNZ_OK;
    }

    size_buf = WRITEBUFFERSIZE;
    buf = (void*)malloc(size_buf);
    if (buf==NULL)
    {
        fprintf(stderr,"Error allocating memory\n");
        return UNZ_INTERNALERROR;
    }

    p = filename_withoutpath = filename_inzip;
    while ((*p) != '\0')
    {
        if (((*p)=='/') || ((*p)=='\\'))
            filename_withoutpath = p+1;
        p++;
    }

    if ((*filename_withoutpath)=='\0')
    {
        if ((*popt_extract_without_path)==0)
        {
            fprintf(stderr,"creating directory: %s\n",getPath2(toFolder,filename_inzip));
            mymkdir(toFolder,filename_inzip);
        }
    }
    else
    {
        char* write_filename;
        int skip=0;

        if ((*popt_extract_without_path)==0)
            write_filename = filename_inzip;
        else
            write_filename = filename_withoutpath;

        err = unzOpenCurrentFilePassword(uf,password);
        if (err!=UNZ_OK)
        {
            fprintf(stderr,"error %d with zipfile in unzOpenCurrentFilePassword\n",err);
        }

        if (((*popt_overwrite)==0) && (err==UNZ_OK))
        {
            char rep=0;
            FILE* ftestexist;
            ftestexist = fopen(getPath2(toFolder, write_filename),"rb");
            if (ftestexist!=NULL)
            {
                fclose(ftestexist);
                do
                {
                    char answer[128];
                    int ret;

                    fprintf(stderr,"The file %s exists. Overwrite ? [y]es, [n]o, [A]ll: ",
						    getPath2(toFolder, write_filename));
                    ret = scanf("%1s",answer);
                    if (ret != 1) 
                    {
                       exit(EXIT_FAILURE);
                    }
                    rep = answer[0] ;
                    if ((rep>='a') && (rep<='z'))
                        rep -= 0x20;
                }
                while ((rep!='Y') && (rep!='N') && (rep!='A'));
            }

            if (rep == 'N')
                skip = 1;

            if (rep == 'A')
                *popt_overwrite=1;
        }

        if ((skip==0) && (err==UNZ_OK))
        {
            fout=fopen(getPath2(toFolder, write_filename),"wb");

            /* some zipfile don't contain directory alone before file */
            if ((fout==NULL) && ((*popt_extract_without_path)==0) &&
                                (filename_withoutpath!=(char*)filename_inzip))
            {
                char c=*(filename_withoutpath-1);
                *(filename_withoutpath-1)='\0';                
                make_mkdir(toFolder, write_filename);                
                *(filename_withoutpath-1)=c;
                fout=fopen(getPath2(toFolder, write_filename),"wb");
            }

            if (fout==NULL)
            {
                fprintf(stderr,"error opening %s\n",getPath2(toFolder, write_filename));
            }
        }

        if (fout!=NULL)
        {
            fprintf(stderr," extracting: %s\n",write_filename);

            do
            {
                err = unzReadCurrentFile(uf,buf,size_buf);
                if (err<0)
                {
                    fprintf(stderr,"error %d with zipfile in unzReadCurrentFile\n",err);
                    break;
                }
                if (err>0)
                    if (fwrite(buf,err,1,fout)!=1)
                    {
                        fprintf(stderr,"error in writing extracted file\n");
                        err=UNZ_ERRNO;
                        break;
                    }
            }
            while (err>0);
            if (fout) {
               fclose(fout);
			}

            if (err==0) {
                change_file_date(getPath2(toFolder,write_filename),file_info.dosDate,
                                 file_info.tmu_date);
               	filecount++;
            }
        }

        if (err==UNZ_OK)
        {
            err = unzCloseCurrentFile (uf);
            if (err!=UNZ_OK)
            {
                fprintf(stderr,"error %d with zipfile in unzCloseCurrentFile\n",err);
            }
        }
        else
            unzCloseCurrentFile(uf); /* don't lose the error */
    }

    free(buf);
    return err;
}


int do_extract_onefile(
    unzFile uf,
	char* toFolder,
	char* filemask, 
    char* filename,  // real filename
    int opt_extract_without_path,
    int opt_overwrite,
    char* password
	)
{
//    int err = UNZ_OK;
    if (unzLocateFile(uf,filename,CASESENSITIVITY)!=UNZ_OK)
    {
    	if (!strstr(filename, "levelinfo.conf")) { 
          fprintf(stderr,"file %s not found in the zipfile\n",filename);
    	}
        return 2;
    }

    if (do_extract_currentfile(uf,toFolder,filemask,
		                       &opt_extract_without_path,
                               &opt_overwrite,
                               password) == UNZ_OK) {
        return 0;
    } else {
        return 1;
    }
}



int do_extract(
    unzFile uf,
	char* toFolder,
    char* filemask,
    int opt_extract_without_path,
    int opt_overwrite,
    char* password)
{
    uLong i;
    unz_global_info gi;
    int err;
//    FILE* fout=NULL;

    err = unzGetGlobalInfo (uf,&gi);
    if (err!=UNZ_OK)
        fprintf(stderr,"error %d with zipfile in unzGetGlobalInfo \n",err);

    filecount = 0;
    dircount  = 0; 
	listFree(&levelInfoList);
	listInit(&levelInfoList);

    for (i=0;i<gi.number_entry;i++)
    {
        if (do_extract_currentfile(uf, toFolder, filemask,
                                   &opt_extract_without_path,
                                   &opt_overwrite,
                                   password) != UNZ_OK)
            break;

        if ((i+1)<gi.number_entry)
        {
            err = unzGoToNextFile(uf);
            if (err!=UNZ_OK)
            {
                fprintf(stderr,"error %d with zipfile in unzGoToNextFile\n",err);
                break;
            }
        }
    }
    fprintf (stderr,"--- extracted dirs %ld files %ld \n", dircount, filecount);
    filecount = 0;
    dircount  = 0; 

	if (!listEmpty(&levelInfoList)) {
      char *pLevInfo;
      char levInfo[512+1];  /* should be enough */
	  if (listGetFirst(&levelInfoList, (void**) &(pLevInfo))) {
		do {  
		  if (pLevInfo[strlen(pLevInfo)-1] == '/') {
		    sprintf(levInfo, "%s%s", pLevInfo, "levelinfo.conf");
		  } else {
		    sprintf(levInfo, "%s/%s", pLevInfo, "levelinfo.conf");
		  }
		  fprintf(stderr, "- try extraxt extra %s\n", levInfo); 
		  if (do_extract_onefile(uf, toFolder, 
		            "levelinfo.conf", // mask
		            levInfo,          // real filename
			        opt_extract_without_path,
                    opt_overwrite,
                    password) == UNZ_OK) {
			fprintf(stderr, "- extraxt extra %s\n", levInfo);
		  } 
		} while (listGetNext(&levelInfoList, (void**) &(pLevInfo)));
	  }
	}
    fprintf (stderr,"--- extra extracted dirs %ld files %ld \n", dircount, filecount);
	listFree(&levelInfoList);

    return 0;
}


int ZUTIL_unzip(char*zipfilename, 
                char*zipfilemask, 
                char*toFolder)
{
    char *password=NULL;
    char filename_try[MAXFILENAME+16] = "";
    int opt_do_extract=1;
    int opt_do_extract_withoutpath=0;
    int opt_overwrite=1;
//    int opt_extractdir=((toFolder != NULL) && (*toFolder != 0));
//    const char *dirname=toFolder;
    
    unzFile uf=NULL;

    if (zipfilename!=NULL)
    {
        strncpy(filename_try, zipfilename,MAXFILENAME-1);
        /* strncpy doesnt append the trailing NULL, of the string is too long. */
        filename_try[ MAXFILENAME ] = '\0';

        uf = unzOpen(zipfilename);
        if (uf==NULL)
        {
            strcat(filename_try,".zip");
            uf = unzOpen(filename_try);
        }
    }

    if (uf==NULL)
    {
        fprintf(stderr,"Cannot open %s or %s.zip\n",zipfilename,zipfilename);
        return 1;
    }
    fprintf(stderr,"--- Zipfile: %s opened\n",filename_try);

	if (opt_do_extract==1)
    {
		/*
        if (opt_extractdir && chdir(dirname)) 
        {
          fprintf(stderr,"Error changing into %s, aborting\n", dirname);
          exit(-1);
        }
		*/

        if ((zipfilemask != NULL) && (zipfilemask[0] != 0))  
            return do_extract(uf, toFolder, zipfilemask,
                              opt_do_extract_withoutpath,
                              opt_overwrite,password);
        else
            return do_extract(uf, toFolder, "*",
                              opt_do_extract_withoutpath,
                              opt_overwrite,password);
    }
    unzCloseCurrentFile(uf);

    return 0;
}


int ZUTIL_unzipAll(char *fromFolder, 
                   char *zipfilename,
                   char *zipfilemask, 
                   char *toFolder)
{
//#HAG#VC
  List_t             dirNameList = {0,0,0,0};
  long               cnt,dcnt;
  char		        *dirNameEntry;

//#HAG#VC
  dcnt = fileScanDir(fromFolder, &dirNameList, NULL);
  if (dcnt < 0)
  {
    Error(ERR_WARN, "cannot read directory '%s'", fromFolder);
    return 1;
  }

  cnt = 0;
  while (cnt < dcnt)	/* loop until last dir entry */
  {
    struct stat file_status;
    char *directory_name = NULL; 
    char *directory_path = NULL; 

	if (!listPosition(&dirNameList, cnt++) ||
        !listGet(&dirNameList, (void**) &dirNameEntry)) {
	  break;
    }

    directory_name = dirNameEntry;
    directory_path = getPath2(fromFolder, directory_name);

    /* skip entries for current and parent directory */
    if (strcmp(directory_name, ".")  == 0 ||
	strcmp(directory_name, "..") == 0)
    {
      free(directory_path);
      continue;
    }

    /* find out if directory entry is itself a directory */
    if (stat(directory_path, &file_status) != 0 ||	/* cannot stat file */
	   (S_ISDIR(file_status.st_mode)) )	/* is a directory */
    {
      free(directory_path);
      continue;
    }


    if (strstr_or_wc(directory_name, 
                        (zipfilename != NULL &&
                         zipfilename[0] != 0) ? zipfilename : 
                         "*.zip" ) == NULL) {
      free(directory_path);
      continue;
    }

    if (ZUTIL_unzip(directory_path, zipfilemask, toFolder) != 0) {
      free(directory_path);
      break;
    }
    
    free(directory_path);
  }

//closedir(dir);
  listFree(&dirNameList); 
  
  return 0;
}

