/***********************************************************
* Rocks'n'Diamonds -- McDuffin Strikes Back!               *
*----------------------------------------------------------*
*               Artsoft Entertainment                      *
*               Holger Schemel                             *
*               Detmolder Strasse 189                      *
*               33604 Bielefeld                            *
*               Germany                                    *
*               e-mail: info@artsoft.org                   *
*----------------------------------------------------------*
* soko.c                                                   *
*----------------------------------------------------------*
* (c) by the owner of the alias/nickname "HerzAusGold"     *
*               e-mail: herzausgold@online.de              *
***********************************************************/


#include "libgame/libgame.h"

#include "events.h"
#include "init.h"
#include "tools.h"
#include "libgame/util.h"
#include "soko.h"

/*---*/

#define KUGEL 	            EL_SOKOBAN_OBJECT
#define ZIEL  	   	    EL_SOKOBAN_FIELD_EMPTY
#define KUGEL_ZIEL 	    EL_SOKOBAN_FIELD_FULL
#define LEER       	    EL_EMPTY
#define FACE       	    -1
#define FACE_ZIEL           EL_SOKOBAN_FIELD_PLAYER
#define KUGEL_PILL 	    EL_SOKOBAN_BOX_PILL
#define KUGEL_PILL_ZIEL	    EL_SOKOBAN_BOX_DEST_PILL
#define PILL_ZIEL           EL_SOKOBAN_DEST_PILL
#define PILL                EL_SOKOBAN_PILL

#define LINKS               left
#define RECHTS              right
#define RAUF                up
#define RUNTER              down

/*---*/

#define FELD(x,y)		Feld[x][y]

#define PLAYERX         pos_x
#define PLAYERY         pos_y

/*---*/

typedef short	     feldEle_t;

typedef struct {
	feldEle_t fd;
	int       x;
	int       y;
} feldEleDraw_t;

/*---*/

static short        findMap[MAX_LEV_FIELDX][MAX_LEV_FIELDY];

static short        pos_x = 0;
static short        pos_y = 0;
static short        left  = 0;
static short        right = 0;
static short        up    = 0;
static short        down  = 0;



/**********************************************************/


#define BADMOVE   (MAX_LEV_FIELDX*MAX_LEV_FIELDY)


static int playerX = 0;
static int playerY = 0;

static List_t pathListN = {0,0,0,0};
static List_t pathListS = {0,0,0,0};
static List_t pathListW = {0,0,0,0};
static List_t pathListE = {0,0,0,0};
static List_t destList  = {0,0,0,0};



static void dumpPath(List_t *pList, char *hStr)
{
#ifdef DEBUG 	
  step_t *pStep;
  printf("---path--- %s\n", hStr);
  if (listGetFirst(pList, (void**) &pStep)) {
    do {
      printf("step: %d push %d\n", pStep->step, pStep->push);
    } while (listGetNext(pList, (void**) &pStep));
  }
#endif  
}

static void ClearMap()
{
  int x,y;
  for (  x=0; x<MAX_LEV_FIELDX; x++ )	{
    for (  y=0; y<MAX_LEV_FIELDY; y++ )	{
      findMap[x][y] = BADMOVE;
    }
  }
}

static void doStep(List_t*pList, short i_step, short i_push)
{
  step_t step;
  step.step = i_step;
  step.push = i_push;
  step.snap = 0;
  listEnter(pList, &step, sizeof(step_t), FALSE);
}

static void FindTarget(int x, int y, int pathLen)
{	
  if (x<0 || y<0 || x>=MAX_LEV_FIELDX || y>=MAX_LEV_FIELDY)
    return;
  if (FELD(x,y)!=LEER && FELD(x,y)!=FACE && FELD(x,y)!=ZIEL && FELD(x,y)!=FACE_ZIEL &&
      !(x == playerX && y == playerY) )
    return;
  if (findMap[x][y] <= pathLen)
    return;
	
  findMap[x][y] = pathLen++;
		
  if((x == playerX) && (y == playerY))
    return;
	
  FindTarget(x - 1, y, pathLen);
  FindTarget(x + 1, y, pathLen);
  FindTarget(x, y - 1, pathLen);
  FindTarget(x, y + 1, pathLen);
}

static bool_t findPath (List_t *pathList, int ax, int ay)
{
  ClearMap();	
  listFree(pathList);
  FindTarget(ax, ay, 0);
  if (findMap[playerX][playerY]!=BADMOVE)
  {
    // save steps
    int cx = playerX;
    int cy = playerY;
    while (findMap[cx][cy])
    {
      if (findMap[cx - 1][cy] == (findMap[cx][cy] - 1)) {
        doStep(pathList, LINKS, FALSE);
        cx--;
      } else if (findMap[cx + 1][cy] == (findMap[cx][cy] - 1)) {
	    doStep(pathList, RECHTS, FALSE);
	    cx++;
      } else if (findMap[cx][cy - 1] == (findMap[cx][cy] - 1)) {
        doStep(pathList, RAUF, FALSE); 
	    cy--;
      } else if (findMap[cx][cy + 1] == (findMap[cx][cy] - 1)) {
        doStep(pathList, RUNTER, FALSE);
	    cy++;
      } else {
        break;  // failure
      } 
    }
  }
  return (!listEmpty(pathList));
}

static void undrawDest() 
{
  feldEleDraw_t *fd;

  if (listGetFirst(&destList, (void**) &fd)) {
	  do {
        FELD(fd->x,fd->y) = fd->fd;
	    DrawLevelField(fd->x, fd->y);	    
	  } while (listGetNext(&destList, (void**) &fd));
  } 
  listFree(&destList);
  listInit(&destList);
  
  listFree(&pathListN);
  listFree(&pathListS);
  listFree(&pathListW);
  listFree(&pathListE);
  listInit(&pathListN);
  listInit(&pathListS);
  listInit(&pathListW);
  listInit(&pathListE);
  
}


static bool_t drawDest(int ax, int ay, int dx, int dy) 
{
  int x=ax+dx;
  int y=ay+dy;
  int retval=FALSE;
  feldEleDraw_t fd;
  if (dx == 0) {
    for (y = ay+dy; (dy!=0 && (y>=0 && y<=MAX_LEV_FIELDY)); y+=dy) {
	  if ((FELD(x,y) == LEER) || (FELD(x,y) == ZIEL)) {
        fd.fd = FELD(x,y);
		fd.x  = x;
		fd.y  = y;
        listEnter(&destList, &fd, sizeof(feldEleDraw_t), FALSE);

		if (FELD(x,y) == ZIEL) {
	      FELD(x,y) = PILL_ZIEL;
		} else {
	      FELD(x,y) = PILL;
		}
	    DrawLevelField(x, y);	    
		retval = TRUE;
	  } else {
	  	break;
	  }
	}	
  }
  if (dy == 0) {
    for (x = ax+dx; (dx!=0 && (x>=0 && x<=MAX_LEV_FIELDX)); x+=dx) {
	  if ((FELD(x,y) == LEER) || (FELD(x,y) == ZIEL)) {
        fd.fd = FELD(x,y);
		fd.x  = x;
		fd.y  = y;
        listEnter(&destList, &fd, sizeof(feldEleDraw_t), FALSE);

		if (FELD(x,y) == ZIEL) {
	      FELD(x,y) = PILL_ZIEL;
		} else {
	      FELD(x,y) = PILL;
		}
	    DrawLevelField(x, y);	    
		retval = TRUE;
	  } else {
	  	break;
	  }
	}	  	
  }
  return retval;
}

void soko (int ax, int ay,int posx, int posy, 
           short le, short ri, short ups, short dwn)
{
   static int sbax = 0;
   static int sbay = 0;
   static int sbrect = FALSE;
   static feldEle_t old_sb;
   feldEle_t sp;
   bool_t findN, findS, findW, findE;
   pos_x = posx;
   pos_y = posy;
   left  = le;
   right = ri;
   up    = ups;
   down  = dwn;
   sp = FELD(ax,ay);
   
   listFree(&goList);
   listInit(&goList);

   if ((FELD(ax,ay) == KUGEL) ||
       (FELD(ax,ay) == KUGEL_ZIEL))
   {	  	
	    if (sbrect) { 
		   // undraw 
		   FELD(sbax,sbay) = old_sb;
		   DrawLevelField(sbax,sbay);

		   undrawDest();
		   SyncDisplay();

		   sbax = 0;
		   sbay = 0;
		   sbrect = FALSE;
#ifdef DEBUG
		   printf ("undraw: x %d y %d feld %d \n", ax,ay,FELD(ax,ay));
#endif
		} 

	    // steps
		playerX = PLAYERX;
		playerY = PLAYERY;	  	    
			
		findN = findPath(&pathListN, ax, ay-1);
		findS = findPath(&pathListS, ax, ay+1);
		findW = findPath(&pathListW, ax-1, ay);
		findE = findPath(&pathListE, ax+1, ay);

		listFree(&destList);
		if (findN || (playerX==ax && playerY==ay-1)) {
			findN = drawDest(ax,ay,0,+1);
		}  
		if (findS || (playerX==ax && playerY==ay+1)) {
			findS = drawDest(ax,ay,0,-1);
		}  
		if (findW || (playerX==ax-1 && playerY==ay)) {
			findW = drawDest(ax,ay,+1,0);
		}  
		if (findE || (playerX==ax+1 && playerY==ay)) {
			findE = drawDest(ax,ay,-1,0);
		}  

		if (findN || findS || findW || findE) {
		  // draw 
		  sbax = ax;
		  sbay = ay;
		  sbrect = TRUE;
#ifdef DEBUG		  
		  printf ("draw: x %d y %d feld %d \n", ax,ay,FELD(ax,ay));
#endif		  

		  old_sb = FELD(ax,ay);
		  if (old_sb == KUGEL) {
		    FELD(ax,ay) = KUGEL_PILL;
		  } else {
			FELD(ax,ay) = KUGEL_PILL_ZIEL;
		  }
		  DrawLevelField(ax,ay);
			
		  SyncDisplay();	 	
		  // dump Steps
		  dumpPath(&pathListN, "N");
		  dumpPath(&pathListS, "S");
		  dumpPath(&pathListW, "W");
		  dumpPath(&pathListE, "E");
		} else {
			if (sbrect) {
			  // undraw 
			  FELD(sbax,sbay) = old_sb;
			  DrawLevelField(sbax,sbay);

			  undrawDest();
			  SyncDisplay();

			  sbax = 0;
			  sbay = 0;
			  sbrect = FALSE;
#ifdef DEBUG			  
			  printf ("undraw: x %d y %d feld %d \n", ax,ay,FELD(ax,ay));
#endif
			}
		}
   } else if ((FELD(ax,ay) == PILL) || 
	      (FELD(ax,ay) == PILL_ZIEL) ||
	      (FELD(ax,ay) == KUGEL_PILL_ZIEL)) {
      int x,y;
      findN = FALSE;
      findS = FALSE;
      findW = FALSE;
      findE = FALSE;
      if ((ax == sbax) && (ay < sbay)) {
	findN = TRUE;
	listMove(&pathListS, &goList);
	listTail(&goList);
	for(y=ay; y<sbay; y++) {
	  doStep(&goList, RAUF, TRUE);
	}
      } else if ((ax == sbax) && (ay > sbay)) {
	findS = TRUE;
	listMove(&pathListN, &goList);
	listTail(&goList);
	for(y=sbay; y<ay; y++) {
	  doStep(&goList, RUNTER, TRUE);
	}
      } else if ((ax < sbax) && (ay == sbay)) {
	findW = TRUE;
	listMove(&pathListE, &goList);
	listTail(&goList);
	for(x=ax; x<sbax; x++) {
	  doStep(&goList, LINKS, TRUE);
	}
      } else if ((ax > sbax) && (ay == sbay)) {
	findE = TRUE;
	listMove(&pathListW, &goList);
	listTail(&goList);
	for(x=sbax; x<ax; x++) {
	  doStep(&goList, RECHTS, TRUE);
	}
      }
      listHead(&goList);

      // dump Steps
      dumpPath(&goList, "-Go-");

      if (sbrect) {
        // undraw 
	FELD(sbax,sbay) = old_sb;
	DrawLevelField(sbax,sbay);

	undrawDest();
	SyncDisplay();

        sbax = 0;
        sbay = 0;
        sbrect = FALSE;
#ifdef DEBUG          
        printf ("undraw: x %d y %d feld %d \n", ax,ay,FELD(ax,ay));
#endif
		}
   } else if (FELD(ax,ay) == LEER) {
      if (sbrect) { 
        // undraw 
        FELD(sbax,sbay) = old_sb;
        DrawLevelField(sbax,sbay);

        undrawDest();
        SyncDisplay();

        sbax = 0;
        sbay = 0;
        sbrect = FALSE;
#ifdef DEBUG          
        printf ("undraw: x %d y %d feld %d \n", ax,ay,FELD(ax,ay));
#endif
      } 

      // steps
      playerX = PLAYERX;
      playerY = PLAYERY;	  	    
	      
      if (findPath(&goList, ax, ay)) {
	listHead(&goList);
      } else {
	listFree(&goList);
      }

   } else {
     if (sbrect) {
       // undraw 
       FELD(sbax,sbay) = old_sb;
       DrawLevelField(sbax,sbay);

       undrawDest();
       SyncDisplay();

       sbax = 0;
       sbay = 0;
       sbrect = FALSE;
#ifdef DEBUG
       printf ("undraw: x %d y %d feld %d \n", ax,ay,FELD(ax,ay));
#endif
     } 
   }
}	  
